;;; The contents of this directory structure is subject to the following
;;; copyright notice:
;;;
;;;	Copyright (c) 2006, 2007 Andrew A. Berlin
;;;
;;;	This program is free software; you can redistribute it and/or
;;;	modify it under the terms of the GNU General Public License as
;;;	published by the Free Software Foundation; either version 2 of
;;;	the License, or (at your option) any later version.
;;;
;;;	This program is distributed in the hope that it will be
;;;	useful, but WITHOUT ANY WARRANTY; without even the implied
;;;	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
;;;	PURPOSE.  See the GNU General Public License for more details.
;;;
;;;	You should have received a copy of the GNU General Public
;;;	License along with this program; if not, write to the Free
;;;	Software Foundation, Inc., 59 Temple Place - Suite 330,
;;;	Boston, MA 02111-1307, USA.

#|
Model of a spherical magnetic probe.  

Given probe diameter, maximum sidewall (translational) force.

Select maximum propulsive velocity of 0.5 cm/sec to ensure good control,
especially near end of tube.   Compute maximum rotation rate of actuator.

circumference-of-probe = distance per rotation
distance per rotation * number of rotations / second = distance per second
number of rotations / second = distance per second / distance per rotation

|#

(define *max-propulsive-velocity* 5e-3) ;;5 mm/sec

(define (compute-max-rotation-rate probe-diameter max-propulsive-velocity)
  (let* ((circumference-of-probe (* pi probe-diameter))
	 (max-rotation-rate (/ max-propulsive-velocity
			       circumference-of-probe)))
    max-rotation-rate))

(define diameters-of-interest (enumerate-interval 0.1e-3 2e-3 0.1e-3))
(define rotation-rates-of-interest
  (map (lambda (diameter)
	 (compute-max-rotation-rate diameter *max-propulsive-velocity*))
       diameters-of-interest))

(plot-xy 'new
	 diameters-of-interest
	 rotation-rates-of-interest)
(smart-gnuplot car
	       cadr
	       (map list
		    diameters-of-interest
		    rotation-rates-of-interest)
	       "Spherical Probe Actuation Rate Limit, based on 0.5mm/sec max probe velocity"
	       "Spherical probe diameter (m)"
	       "Maximum rotation rate (Cycles/sec)"
	       #t
	       )
