;;; The contents of this directory structure is subject to the following
;;; copyright notice:
;;;
;;;	Copyright (c) 2006, 2007 Andrew A. Berlin
;;;
;;;	This program is free software; you can redistribute it and/or
;;;	modify it under the terms of the GNU General Public License as
;;;	published by the Free Software Foundation; either version 2 of
;;;	the License, or (at your option) any later version.
;;;
;;;	This program is distributed in the hope that it will be
;;;	useful, but WITHOUT ANY WARRANTY; without even the implied
;;;	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
;;;	PURPOSE.  See the GNU General Public License for more details.
;;;
;;;	You should have received a copy of the GNU General Public
;;;	License along with this program; if not, write to the Free
;;;	Software Foundation, Inc., 59 Temple Place - Suite 330,
;;;	Boston, MA 02111-1307, USA.

;;;Define the catalogue of magnets 

(define magnet-1mmx3mm 
  (let ((mass (* (/ 3 4) ;;scale up mass to account for extra material in 3mm magnet
		 2.2679e-5)) ;;Kg ;;2.2679e-5 is # from 4mm length magnet
	(Br (gauss->tesla 12510.0)) ;;Tesla
	(len (mm->m 3.0))
	(diameter (mm->m 1.0)))
    (make-cylindrical-magnet 'magnet-1mmx3mm
			     mass
			     len
			     diameter
			     Br)))


(define magnet-Gaussboys-1mmx4mm 
  (let ((mass 2.2679e-5) ;;Kg
	(Br (gauss->tesla 12510.0)) ;;Tesla
	(len (mm->m 4.0))
	(diameter (mm->m 1.0)))
    (make-cylindrical-magnet 'Gaussboys-magnet-1mmx4mm
			     mass
			     len
			     diameter
			     Br)))

(define magnet-2mmx4mm
  (let* ((mass-of-1mmx4mm 2.2679e-5) ;;Kg
	 (diameter-of-1mmx4mm (mm->m 1.0))
	 (Br (gauss->tesla 12510.0)) ;;Tesla
	 (len (mm->m 4.0))
	 (diameter (mm->m 2.0))
	 (mass-of-2mmx4mm (* mass-of-1mmx4mm
			     (/ (square (/ diameter 2))
				(square (/ diameter-of-1mmx4mm 2))))))
    (make-cylindrical-magnet 'magnet-2mmx4mm
			     mass-of-2mmx4mm
			     len
			     diameter
			     Br)))

(define magnet-1mmx7mm 
  (let ((mass (* (/ 7 4) ;;scale up mass to account for extra material in 7mm magnet
		 2.2679e-5)) ;;Kg ;;2.2679e-5 is # from 4mm length magnet
	(Br (gauss->tesla 12510.0)) ;;Tesla
	(len (mm->m 7.0))
	(diameter (mm->m 1.0)))
    (make-cylindrical-magnet 'magnet-1mmx7mm
			     mass
			     len
			     diameter
			     Br)))

(define magnet-1mmx10mm 
  (let ((mass (* (/ 10 4) ;;scale up mass to account for extra material in 10mm magnet
		 2.2679e-5)) ;;Kg ;;2.2679e-5 is # from 4mm length magnet
	(Br (gauss->tesla 12510.0)) ;;Tesla
	(len (mm->m 10.0))
	(diameter (mm->m 1.0)))
    (make-cylindrical-magnet 'magnet-1mmx10mm
			     mass
			     len
			     diameter
			     Br)))

(define magnet-1mmx20mm 
  (let ((mass (* (/ 20 4) ;;scale up mass to account for extra material in 20mm magnet
		 2.2679e-5)) ;;Kg ;;2.2679e-5 is # from 4mm length magnet
	(Br (gauss->tesla 12510.0)) ;;Tesla
	(len (mm->m 20.0))
	(diameter (mm->m 1.0)))
    (make-cylindrical-magnet 'magnet-1mmx20mm
			     mass
			     len
			     diameter
			     Br)))

(define magnet-Gaussboys-large-37mmx37mm
  (let ((len (mm->m 37))
	(diameter (mm->m 37)) ;;not a typo - both length and diameter are 37mm
	(mass 0.3) ;Kg
	(Br 1.2650) ;Tesla
	)
    (make-cylindrical-magnet 'Gaussboys-large-37mmx37mm
			     mass
			     len
			     diameter
			     Br)))

