;;; The contents of this directory structure is subject to the following
;;; copyright notice:
;;;
;;;	Copyright (c) 2006, 2007 Andrew A. Berlin
;;;
;;;	This program is free software; you can redistribute it and/or
;;;	modify it under the terms of the GNU General Public License as
;;;	published by the Free Software Foundation; either version 2 of
;;;	the License, or (at your option) any later version.
;;;
;;;	This program is distributed in the hope that it will be
;;;	useful, but WITHOUT ANY WARRANTY; without even the implied
;;;	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
;;;	PURPOSE.  See the GNU General Public License for more details.
;;;
;;;	You should have received a copy of the GNU General Public
;;;	License along with this program; if not, write to the Free
;;;	Software Foundation, Inc., 59 Temple Place - Suite 330,
;;;	Boston, MA 02111-1307, USA.

;;Define an unusual shape (discontinuous) fallopian tube that forms a grid.

(define offsetgrid-x-distance (in->m 3))
(define offsetgrid-ypitch (cm->m 1.0))
(define offsetgrid-num-y-rows 7)
(define offsetgrid-zpitch offsetgrid-ypitch) ;;keep it symmetrical for now
(define offsetgrid-num-z-rows offsetgrid-num-y-rows) ;;symmetrical for now
(define offsetgrid-startpt (make-3point 0 (cm->m -3) (cm->m -3)))
(define offsetgrid-endpt
  (3point-+ offsetgrid-startpt
	    (make-3point offsetgrid-x-distance
			 (* offsetgrid-ypitch offsetgrid-num-y-rows)
			 (* offsetgrid-zpitch offsetgrid-num-z-rows))))

(define offsetgrid-L
  (* offsetgrid-x-distance
     offsetgrid-num-y-rows
     offsetgrid-num-z-rows))

(define gridtube-topology
  (make-fallopian-tube 'gridtube
		       offsetgrid-startpt
		       offsetgrid-endpt
		       offsetgrid-3d-position
		       offsetgrid-3d-orientation
		       offsetgrid-L
		       offsetgrid-x-distance
		       offsetgrid-num-y-rows
		       offsetgrid-ypitch
		       offsetgrid-zpitch
		       ))


(define *or-target* (make-3point (in->m 0)
				 (in->m 0)
				 (in->m 0)))
(define a1 (make-3point (+ (in->m 8) (/ (mm->m 37) 2))
			(* -.75 (mm->m 37)) ;;.75 actuator lengths to over
			(* -.75 (mm->m 37))))
(define a2 (3point-+ a1
		     (make-3point 0 ;;same x as a1
				  (* 1.5 (mm->m 37)) ;;1.5 magnet lengths over
				  (* 1.5 (mm->m 37)) ;;1.5 magnet lengths up
				  )))
(define a3 (3point-+ a1
		     (make-3point 0 ;;same x as a1
				  (* 1.5 (mm->m 37)) ;;1.5 magnet lengths over
				  (* 0 (mm->m 37)) ;;same z as a1
				  )))
(define a4 (3point-+ a1
		     (make-3point 0 ;;same x as a1
				  0 ;;same y as a1
				  (* 1.5 (mm->m 37)) ;;1.5 magnet lengths up
				  )))


(define gridtube-world-4
  (define-world 'gridtube-world-4
    (topology gridtube-topology)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-Gaussboys-1mmx4mm))
    (actuator 'actuator1
	      (location a1)
	      (orientation-target *or-target*)
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    
    (actuator 'actuator2
	      (location a2)
	      (orientation-target *or-target*)
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))

    (actuator 'actuator3
	      (location a3)
	      (orientation-target *or-target*)
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))

    (actuator 'actuator4
	      (location a4)
	      (orientation-target *or-target*)
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))

  ))


(define gridtube-world-1
  (define-world 'gridtube-world-1
    (topology gridtube-topology)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-Gaussboys-1mmx4mm))
    (actuator 'actuator1
	      (location (make-3point (3point-x a1)
				     0
				     0))
	      (orientation-target *or-target*)
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
  ))

(define gridtube-world-1-stacked
  (define-world 'gridtube-world-1-stacked
    (topology gridtube-topology)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-Gaussboys-1mmx4mm))
    (actuator 'actuator1
	      (location (make-3point (3point-x a1)
				     0
				     0))
	      (orientation-target *or-target*)
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ;;second actuator right behind the first, effectively extending length
    (actuator 'actuator2 
	      (location (make-3point (+ (3point-x a1) (mm->m 37))
				     0
				     0))
	      (orientation-target *or-target*)
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
  ))

(define gridtube-world-2
  (define-world 'gridtube-world-2
    (topology gridtube-topology)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-Gaussboys-1mmx4mm))
    (actuator 'actuator1
	      (location (make-3point (3point-x a1)
				     0
				     (3point-z a1)))
	      (orientation-target *or-target*)
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    (actuator 'actuator2
	      (location (make-3point (3point-x a1)
				     0
				     (3point-z a2)))
	      (orientation-target *or-target*)
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
  ))



;;(describe-world gridtube-world)

(set! dt 0.001) ;;how often to print, in seconds
(set! *run-until* 0.1)

(run-simulator gridtube-world-4 *run-until* #t) ;#t says to print state at each dt


(set! *default-heatfile-alpha* 0.07)
;(write-files! gridtube-world-4) ;;plot-menu will write-files
(format #t "~%~a" '(plot-menu gridtube-world-4)) 
(plot-menu gridtube-world-4)
