;;; The contents of this directory structure is subject to the following
;;; copyright notice:
;;;
;;;	Copyright (c) 2006, 2007 Andrew A. Berlin
;;;
;;;	This program is free software; you can redistribute it and/or
;;;	modify it under the terms of the GNU General Public License as
;;;	published by the Free Software Foundation; either version 2 of
;;;	the License, or (at your option) any later version.
;;;
;;;	This program is distributed in the hope that it will be
;;;	useful, but WITHOUT ANY WARRANTY; without even the implied
;;;	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
;;;	PURPOSE.  See the GNU General Public License for more details.
;;;
;;;	You should have received a copy of the GNU General Public
;;;	License along with this program; if not, write to the Free
;;;	Software Foundation, Inc., 59 Temple Place - Suite 330,
;;;	Boston, MA 02111-1307, USA.

(declare (usual-integrations))

;;;in Gnuplot, do splot 'filename.txt' using 1:2:3:4 with lines palette
;;;to do 4-D colorized plot


;;dataset is a list of vectors
;;v1, v2, and v3 are access functions
(define (plot-dataset-csv dataset . args)
  (define (plot-dataset-item item args)
    (if (null? args)
	(newline)
	(let ((accessor-function (car args)))
	  (format #t "~a " (* 1.0 (accessor-function item)))
	  (plot-dataset-item item (cdr args)))))
  (for-each (lambda (item) (plot-dataset-item item args))
	    dataset))

;;Any changes to this plot order must also be reflectedin the gnuplot plot generation scripts
(define (plot-fallopian-tube-profile-csv world #!optional increment)
  (if (default-object? increment)
      (set! increment *plot-fallopian-tube-default-increment*))
  (let* ((forces-profile
	  (make-fallopian-tube-forces-profile world increment)))
    (plot-fallopian-tube-profile-dataset-csv forces-profile)))

(define (plot-fallopian-tube-profile-dataset-csv forces-profile)
  (plot-dataset-csv
   forces-profile
   (lambda (entry)
     (3point-x (fallopian-tube-force-profile-3pt-location entry))) ;1
   (lambda (entry)
     (3point-y (fallopian-tube-force-profile-3pt-location entry))) ;2
   (lambda (entry)
     (3point-z (fallopian-tube-force-profile-3pt-location entry))) ;3
   fallopian-tube-force-profile-propulsive-accel ;4
   fallopian-tube-force-profile-max-sidewall-accel ;5
   (lambda (entry) ;;6
     (3point-magnitude
      (fallopian-tube-force-profile-Torque entry)))
   fallopian-tube-force-profile-alpha	;7
   fallopian-tube-force-profile-rotation-force-mag ;8
   fallopian-tube-force-profile-rotation-accel ;9
   ))
    
;;Any changes to this plot order must also be reflectedin the gnuplot plot generation scripts
(define (plot-simulation-results-csv world)
  (if (> (length (world-probes world)) 1)
      (error "plot-simulation-results only handles one probe for now"))
  (let* ((fts (map (lambda (state)
		     (let* ((alpha (state-pos state))
			    (time (state-time state))
			    (vel (state-vel state))
			    (ft (compute-fallopian-tube-forces-at-alpha world alpha time vel)))
		       ft))
		   (world-state-history world))))
    (plot-dataset-csv
     fts
     fallopian-tube-force-profile-time	;1
     fallopian-tube-force-profile-alpha ;2
     fallopian-tube-force-profile-vel	;3
     (lambda (ft) (3point-x (fallopian-tube-force-profile-3pt-location ft))) ;4
     (lambda (ft) (3point-y (fallopian-tube-force-profile-3pt-location ft))) ;5
     (lambda (ft) (3point-z (fallopian-tube-force-profile-3pt-location ft))) ;6
     fallopian-tube-force-profile-propulsive-force-mag ;7
     fallopian-tube-force-profile-propulsive-accel ;8
     fallopian-tube-force-profile-max-sidewall-force ;9
     fallopian-tube-force-profile-max-sidewall-accel ;10
     (lambda (entry)			;11
       (3point-magnitude
	(fallopian-tube-force-profile-Torque entry)))
     fallopian-tube-force-profile-translation-force-mag ;12
     fallopian-tube-force-profile-translation-accel ;13
     fallopian-tube-force-profile-rotation-force-mag ;14
     fallopian-tube-force-profile-rotation-accel) ;15
    ))



(define (plot-actuator-positive-monopole-locations world)
  (let* ((monopoles (map (lambda (a)
			   (if (positive? (monopole-strength (actuator-monopole1 a)))
			       (actuator-monopole1 a)
			       (actuator-monopole2 a)))
			 (world-actuators world))))
    (plot-dataset-csv monopoles
		      (lambda (m) (3point-x (monopole-location m)))
		      (lambda (m) (3point-y (monopole-location m)))
		      (lambda (m) (3point-z (monopole-location m)))
		      (lambda (m) 0)) ;;0 included for gnuplot to set position (i.e. color) for 4-D plots
    ))

(define (plot-actuator-negative-monopole-locations world)
  (let* ((monopoles (map (lambda (a)
			   (if (positive? (monopole-strength (actuator-monopole1 a)))
			       (actuator-monopole2 a)
			       (actuator-monopole1 a)))
			 (world-actuators world))))
    (plot-dataset-csv monopoles
		      (lambda (m) (3point-x (monopole-location m)))
		      (lambda (m) (3point-y (monopole-location m)))
		      (lambda (m) (3point-z (monopole-location m)))
		      (lambda (m) 0))  ;;0 included for gnuplot to set position (i.e. color) for 4-D plots
    ))


(define plot-menu-items)
(set! plot-menu-items '())

(define plot-menu-last-win (make-default-object))

(define-structure plot-menu-entry text proc)
(define (register-plot-menu-item! text proc)
  (set! plot-menu-items
	(append  plot-menu-items
		 (list (make-plot-menu-entry text proc))))
  (format #t "~%register-plot-menu-item!: ~a added to menu" text)
  )

(define (plot-menu world)
  (write-files! world)
  (let loop ()
    (let ((states (world-state-history world)))
      (for-each (lambda (item no)
		  (format #t "~%~a.  ~a"
			  no
			  (plot-menu-entry-text item)
			  ))
		plot-menu-items
		(enumerate-interval 1 (length plot-menu-items)))
      (format #t "~%Please make selection by number: ~%")
      (let* ((selection-no (read))
	     (menu-entry (list-ref plot-menu-items (-1+ selection-no)))
	     (proc (plot-menu-entry-proc menu-entry)))
	(if (eq? proc exit-menu)
	    (format #t "~%Exiting plot-menu system")
	    (begin (set! plot-menu-last-win (proc world states))
		   (loop)))))))
  
(define (enumerate-interval start end #!optional increment)
  (if (default-object? increment)
      (set! increment 1))
  (let loop ((i start))
    (if (> i end)
	'()
	(cons i (loop (+ i increment))))))

(define (exit-menu world states)
  #f)

(define (plot-a-then-b a b #!optional c win)
  (if (not (default-object? win))
      (x-graphics/set-foreground-color win "green"))
  (let ((newwin (a win)))
    (x-graphics/set-foreground-color newwin "blue")
    (let ((newerwin (b newwin)))
      (if (not (default-object? c))
	  (begin
	    (x-graphics/set-foreground-color newerwin "yellow")
	    (c newerwin))
	  newerwin))))

(register-plot-menu-item! "Plot Position" (lambda (world states) (plot-pos states)))
(register-plot-menu-item! "Plot Velocity" (lambda (world states) (plot-vel states)))
(register-plot-menu-item! "Plot Position then Velocity"
			  (lambda (world states)
			    (plot-a-then-b
			     (lambda (win) (plot-pos states win))
			     (lambda (win) (plot-vel states win)))))
(register-plot-menu-item! "Plot Velocity then Position"
			  (lambda (world states)
			    (plot-a-then-b
			     (lambda (win) (plot-vel states win))
			     (lambda (win) (plot-pos states win)))))
(register-plot-menu-item! "Plot Phase Portrait"
			  (lambda (world states) (plot-phase-portrait states)))
(register-plot-menu-item! "Plot Actuator X coords on last window"
			  (lambda (world states) (plot-actuators-x world plot-menu-last-win)))
(register-plot-menu-item! "Plot propulsion acceleration profile"
			  (lambda (world states) (plot-accel-profile world)))
(register-plot-menu-item! "Plot sidewall acceleration profile"
			  (lambda (world states) (plot-sidewall-accel-profile world)))
(register-plot-menu-item! "Plot propulsion acceleration then sidewall accel profile"
			  (lambda (world states)
			    (plot-a-then-b
			     (lambda (win) (plot-accel-profile world win))
			     (lambda (win) (plot-sidewall-accel-profile world win)))))
(register-plot-menu-item! "Plot sidewall acceleration then propulsion accel profile" 
			  (lambda (world states)
			    (plot-a-then-b
			     (lambda (win) (plot-sidewall-accel-profile world win))
			     (lambda (win) (plot-accel-profile world win)))))
(register-plot-menu-item! "Mouse on last window plotted"
			  (lambda (world states) (mouse plot-menu-last-win)))
(register-plot-menu-item! "Describe World"
			  (lambda (world states) (describe-world world)))

(register-plot-menu-item! "Gnuplot 3D Tube Topology"
			  (lambda (world states)
			    (format pipeport "load 'gnuplot-draw-tube'~%")
			    (flush-output pipeport)))

(register-plot-menu-item! "Gnuplot 3D Tube Topology with Accel Heatmap"
			  (lambda (world states)
			    (format pipeport "load 'gnuplot-draw-tube-with-accel-heatmap'~%")
			    (flush-output pipeport)))

(register-plot-menu-item! "Gnuplot 3D Accel Profile"
			  (lambda (world states)
			    (format pipeport "load 'gnuplot-draw-accel-profile'~%")
			    (flush-output pipeport)))

(register-plot-menu-item! "Gnuplot 3D Accel Profile with Heatmap"
			  (lambda (world states)
			    (format pipeport "load 'gnuplot-draw-accel-profile-with-heatmap'~%")
			    (flush-output pipeport)))
(register-plot-menu-item! "Gnuplot 3D SideAccel Profile"
			  (lambda (world states)
			    (format pipeport "load 'gnuplot-draw-sideaccel-profile'~%")
			    (flush-output pipeport)))
(register-plot-menu-item! "Gnuplot 3D SideAccel Profile with Heatmap"
			  (lambda (world states)
			    (format pipeport "load 'gnuplot-draw-sideaccel-profile-with-heatmap'~%")
			    (flush-output pipeport)))
(register-plot-menu-item! "Gnuplot 3D Torque Profile"
			  (lambda (world states)
			    (format pipeport "load 'gnuplot-draw-torque-profile'~%")
			    (flush-output pipeport)))
(register-plot-menu-item! "Gnuplot 3D Results and Tube"
			  (lambda (world states)
			    (format pipeport "load 'gnuplot-draw-results-and-tube'~%")
			    (flush-output pipeport)))
(register-plot-menu-item! "Gnuplot 3D Results Only"
			  (lambda (world states)
			    (format pipeport "load 'gnuplot-draw-results-only'~%")
			    (flush-output pipeport)))
(register-plot-menu-item! "Gnuplot Heatmap Accel"
			  (lambda (world states)
			    (format pipeport "load 'gnuplot-draw-accel-heatmap'~%")
			    (flush-output pipeport)))
(register-plot-menu-item! "Gnuplot Heatmap SideAccel"
			  (lambda (world states)
			    (format pipeport "load 'gnuplot-draw-sideaccel-heatmap'~%")
			    (flush-output pipeport)))
(register-plot-menu-item! "Gnuplot Heatmap Torque"
			  (lambda (world states)
			    (format pipeport "load 'gnuplot-draw-torque-heatmap'~%")
			    (flush-output pipeport)))
(register-plot-menu-item!
 "Specify Alpha for Heatmaps "
 (lambda (world states)
   (format #t "~%Alpha currently set to: ~a.  Specify new value for alpha: "
	   *default-heatfile-alpha*)
   (let ((alpha (read)))
     (if (number? alpha)
	 (begin (format #t "~%Updating heatmap alpha to ~a - please wait..." alpha)
		(set! *default-heatfile-alpha* alpha)
		(write-heatfiles! world alpha)
		(format #t "~%Alpha updated to: ~a"
			alpha))
	 (format #t "~%Alpha NOT updated.  Input not recognized.")
	 ))))

(register-plot-menu-item!
 "Specify Number of Heatmaps "
 (lambda (world states)
   (format #t "~%Number of Heatmaps currently set to: ~a.  Specify new number of heatmaps: "
	   *default-heatfile-number-of-maps*)
   (let ((nummaps (read)))
     (if (number? nummaps)
	 (begin (format #t "~%Updating heatmap number of maps to ~a - please wait..." nummaps)
		(set! *default-heatfile-number-of-maps* nummaps)
		(write-heatfiles! world)
		(format #t "~%Heatmap number of maps updated to: ~a"
			nummaps))
	 (format #t "~%Heatmap number of maps NOT updated.  Input not recognized.")
	 ))))									  
(register-plot-menu-item! "Exit menu system" exit-menu)
(format #t "~%~%To start menu system, do (plot-menu world)")
